import { LitElement, html, css } from 'lit';
import { property } from 'lit/decorators.js';
import { UUIHorizontalShakeKeyframes, UUIHorizontalShakeAnimationValue } from '@umbraco-ui/uui-base/lib/animations';
import { LabelMixin } from '@umbraco-ui/uui-base/lib/mixins';
import { InterfaceLookDefaultValue } from '@umbraco-ui/uui-base/lib/types';

var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __decorateClass = (decorators, target, key, kind) => {
  var result = kind > 1 ? void 0 : kind ? __getOwnPropDesc(target, key) : target;
  for (var i = decorators.length - 1, decorator; i >= 0; i--)
    if (decorator = decorators[i])
      result = (kind ? decorator(target, key, result) : decorator(result)) || result;
  if (kind && result)
    __defProp(target, key, result);
  return result;
};
class UUIButtonElement extends LabelMixin("", LitElement) {
  constructor() {
    super();
    this.disabled = false;
    this.look = InterfaceLookDefaultValue;
    this.compact = false;
    this.addEventListener("click", this.onHostClick);
  }
  onHostClick(e) {
    if (this.disabled) {
      e.preventDefault();
      e.stopImmediatePropagation();
    }
  }
  render() {
    return html`
      <button ?disabled=${this.disabled} aria-label="${this.label}">
        ${this.renderLabel()}
      </button>
    `;
  }
}
UUIButtonElement.styles = [
  UUIHorizontalShakeKeyframes,
  css`
      :host {
        position: relative;
        display: inline-block;
        margin-left: calc(var(--uui-button-merge-border-left, 0) * -1px);
        --uui-button-slot-padding-l-factor: 3;
        --uui-button-slot-padding-r-factor: 3;
        background-color: var(--uui-interface-surface,#FEFEFE);
      }

      :host([compact]) {
        --uui-button-slot-padding-l-factor: 1;
        --uui-button-slot-padding-r-factor: 1;
      }

      button {
        height: 100%;
        min-height: var(
          --uui-button-height,
          calc(var(--uui-button-base-unit, var(--uui-size-base-unit,6px)) * 6)
        );
        width: 100%;
        padding: 0;
        text-align: center;
        vertical-align: middle;
        box-shadow: none;
        border-width: var(--uui-button-border-width, 1px);
        border-style: solid;
        border-color: var(
          --uui-button-border-color,
          var(--uui-interface-surface,#FEFEFE)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-size-border-radius,3px)
        );
        cursor: pointer;
        font-weight: var(
          --uui-button-font-weight,
          var(--uui-interface-font-weight,400)
        );
        font-size: inherit;
        font-family: inherit;

        background-color: var(
          --uui-button-background-color,
          var(--uui-interface-surface,#FEFEFE)
        );
        color: var(--uui-button-contrast, var(--uui-interface-contrast,#060606));

        transition: background-color 80ms, border-color 80ms, color 80ms;
      }
      button:hover {
        background-color: var(
          --uui-button-background-color-hover,
          var(--uui-interface-surface-hover,rgb(250, 250, 250))
        );
        border-color: var(
          --uui-button-border-color-hover,
          var(--uui-interface-surface-hover,rgb(250, 250, 250))
        );
        color: var(
          --uui-button-contrast-hover,
          var(--uui-interface-contrast-hover,#1B264F)
        );
      }

      button[disabled] {
        background-color: var(
          --uui-button-background-color-disabled,
          var(--uui-interface-surface-disabled,rgb(228, 228, 228))
        );
        color: var(
          --uui-button-contrast-disabled,
          var(--uui-interface-contrast-disabled,rgb(115, 113, 110))
        );
        cursor: default;
      }
      button[disabled]:active {
        -webkit-animation: ${UUIHorizontalShakeAnimationValue};
                animation: ${UUIHorizontalShakeAnimationValue};
      }

      button > .label {
        display: block;
        padding: 0
          calc(
            (
              var(--uui-button-base-unit, var(--uui-size-base-unit,6px)) *
                var(--uui-button-slot-padding-r-factor)
            )
          )
          0
          calc(
            (
              var(--uui-button-base-unit, var(--uui-size-base-unit,6px)) *
                var(--uui-button-slot-padding-l-factor)
            )
          );
      }

      /* LOOKS */

      :host([look='primary']) button {
        background-color: var(--uui-look-primary-surface,#1B264F);
        color: var(--uui-look-primary-contrast,#FEFEFE);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-primary-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-primary-border-radius, var(--uui-size-border-radius,3px))
        );
        border-color: var(--uui-look-primary-border,#1B264F);
        font-weight: var(--uui-look-primary-font-weight,700);
      }
      :host([look='primary']) button:hover {
        background-color: var(--uui-look-primary-surface-hover,#3544B1);
        color: var(--uui-look-primary-contrast-hover,#FEFEFE);
        border-color: var(--uui-look-primary-border-hover,#3544B1);
      }
      :host([look='primary']) button[disabled] {
        background-color: var(--uui-look-primary-surface-disabled,rgb(115, 120, 130));
        color: var(--uui-look-primary-contrast-disabled,rgb(228, 228, 228));
        border-color: var(--uui-look-primary-border-disabled,rgb(115, 120, 130));
      }

      :host([look='secondary']) button {
        background-color: var(--uui-look-secondary-surface,#E2DAD4);
        color: var(--uui-look-secondary-contrast,#1B264F);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-secondary-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-secondary-border-radius, var(--uui-size-border-radius,3px))
        );
        border-color: var(--uui-look-secondary-border,#E2DAD4);
        font-weight: var(--uui-look-secondary-font-weight,500);
      }
      :host([look='secondary']) button:hover {
        background-color: var(--uui-look-secondary-surface-hover,rgb(243, 239, 237));
        color: var(--uui-look-secondary-contrast-hover,#3544B1);
        border-color: var(--uui-look-secondary-border-hover,rgb(243, 239, 237));
      }
      :host([look='secondary']) button[disabled] {
        background-color: var(--uui-look-secondary-surface-disabled,rgb(207, 207, 207));
        color: var(--uui-look-secondary-contrast-disabled,rgb(106, 117, 154));
        border-color: var(--uui-look-secondary-border-disabled,rgb(207, 207, 207));
      }

      :host([look='outline']) button {
        background-color: var(--uui-look-outline-surface,transparent);
        color: var(--uui-look-outline-contrast,#1B264F);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-outline-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-outline-border-radius, var(--uui-size-border-radius,3px))
        );
        border-color: var(--uui-look-outline-border,#1B264F);
        font-weight: var(--uui-look-outline-font-weight,700);
      }
      :host([look='outline']) button:hover {
        background-color: var(--uui-look-outline-surface-hover,transparent);
        color: var(--uui-look-outline-contrast-hover,#3544B1);
        border-color: var(--uui-look-outline-border-hover,#3544B1);
      }
      :host([look='outline']) button[disabled] {
        background-color: var(--uui-look-outline-surface-disabled,transparent);
        color: var(--uui-look-outline-contrast-disabled,rgb(106, 117, 154));
        border-color: var(--uui-look-outline-border-disabled,rgb(106, 117, 154));
      }

      :host([look='placeholder']) button {
        background-color: var(--uui-look-placeholder-surface,transparent);
        color: var(--uui-look-placeholder-contrast,#1B264F);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-placeholder-border-style, dashed)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(
            --uui-look-placeholder-border-radius,
            var(--uui-size-border-radius,3px)
          )
        );
        border-color: var(--uui-look-placeholder-border,#1B264F);
        font-weight: var(--uui-look-placeholder-font-weight,400);
      }
      :host([look='placeholder']) button:hover {
        background-color: var(--uui-look-placeholder-surface-hover,transparent);
        color: var(--uui-look-placeholder-contrast-hover,#3544B1);
        border-color: var(--uui-look-placeholder-border-hover,#3544B1);
      }
      :host([look='placeholder']) button[disabled] {
        background-color: var(--uui-look-placeholder-surface-disabled,transparent);
        color: var(--uui-look-placeholder-contrast-disabled,rgb(106, 117, 154));
        border-color: var(--uui-look-placeholder-border-disabled,rgb(106, 117, 154));
      }

      :host([look='positive']) button {
        background-color: var(--uui-look-positive-surface,#25aa60);
        color: var(--uui-look-positive-contrast,#FEFEFE);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-positive-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-positive-border-radius, var(--uui-size-border-radius,3px))
        );
        border-color: var(--uui-look-positive-border,#25aa60);
        font-weight: var(--uui-look-positive-font-weight,700);
      }
      :host([look='positive']) button:hover {
        background-color: var(--uui-look-positive-surface-hover,rgb(43, 197, 112));
        color: var(--uui-look-positive-contrast-hover,#FEFEFE);
        border-color: var(--uui-look-positive-border-hover,rgb(43, 197, 112));
      }
      :host([look='positive']) button[disabled] {
        background-color: var(--uui-look-positive-surface-disabled,rgb(89, 117, 102));
        color: var(--uui-look-positive-contrast-disabled,#FEFEFE);
        border-color: var(--uui-look-positive-border-disabled,rgb(89, 117, 102));
      }

      :host([look='warning']) button {
        background-color: var(--uui-look-warning-surface,#FAD634);
        color: var(--uui-look-warning-contrast,#060606);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-warning-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-warning-border-radius, var(--uui-size-border-radius,3px))
        );
        border-color: var(--uui-look-warning-border,#FAD634);
        font-weight: var(--uui-look-warning-font-weight,700);
      }
      :host([look='warning']) button:hover {
        background-color: var(--uui-look-warning-surface-hover,rgb(251, 224, 101));
        color: var(--uui-look-warning-contrast-hover,#060606);
        border-color: var(--uui-look-warning-border-hover,rgb(251, 224, 101));
      }
      :host([look='warning']) button[disabled] {
        background-color: var(--uui-look-warning-surface-disabled,rgb(166, 160, 133));
        color: var(--uui-look-warning-contrast-disabled,#060606);
        border-color: var(--uui-look-warning-border-disabled,rgb(166, 160, 133));
      }

      :host([look='danger']) button {
        background-color: var(--uui-look-danger-surface,#D42054);
        color: var(--uui-look-danger-contrast,#FEFEFE);
        border-style: var(
          --uui-button-border-style,
          var(--uui-look-danger-border-style, solid)
        );
        border-radius: var(
          --uui-button-border-radius,
          var(--uui-look-danger-border-radius, var(--uui-size-border-radius,3px))
        );
        border-color: var(--uui-look-danger-border,#D42054);
        font-weight: var(--uui-look-danger-font-weight,700);
      }
      :host([look='danger']) button:hover {
        background-color: var(--uui-look-danger-surface-hover,rgb(226, 60, 107));
        color: var(--uui-look-danger-contrast-hover,#FEFEFE);
        border-color: var(--uui-look-danger-border-hover,rgb(226, 60, 107));
      }
      :host([look='danger']) button[disabled] {
        background-color: var(--uui-look-danger-surface-disabled,rgb(133, 107, 114));
        color: var(--uui-look-danger-contrast-disabled,#FEFEFE);
        border-color: var(--uui-look-danger-border-disabled,rgb(133, 107, 114));
      }
    `
];
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIButtonElement.prototype, "disabled", 2);
__decorateClass([
  property({ reflect: true })
], UUIButtonElement.prototype, "look", 2);
__decorateClass([
  property({ type: Boolean, reflect: true })
], UUIButtonElement.prototype, "compact", 2);

export { UUIButtonElement };
